@@ -184,7 +184,9 @@ class MatriceProxyInterface:
             input_url_value = input_url or inputUrl
             if input_url_value:
                 try:
-                    async with httpx.AsyncClient() as client:
+                    # Use timeout and error handling for URL downloads
+                    timeout = httpx.Timeout(60.0)  # 10 second timeout
+                    async with httpx.AsyncClient(timeout=timeout) as client:
                         response = await client.get(input_url_value)
                         response.raise_for_status()  # Raise exception for HTTP errors
                         input1 = response.content
